package controlers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.Resource;
import javax.enterprise.context.SessionScoped;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@SessionScoped
@Named(value = "Application")
public class Application implements Serializable {

    @PersistenceContext(unitName = "HierarchyPU")
    private EntityManager entityManager;

    @Resource
    private javax.transaction.UserTransaction utx;

    private List<Zaznam> zoznamZaznamov;
    private List<Kategoria> zoznamKategorii;

    private Zaznam viewZaznam;
    private Kategoria viewKategoria;
    private String newLabel = "";
    private int newID = 0;
    private int deleteID = 0;
    
    private List<Kategoria> searchKategorie = new ArrayList();
    private List<Kategoria> mozneKategorie; 
    private List<Zaznam> filtrovaneZaznamy = new ArrayList();

    public List<Zaznam> getFiltrovaneZaznamy() {
        return filtrovaneZaznamy;
    }

    public void setFiltrovaneZaznamy(List<Zaznam> filtrovaneZaznamy) {
        this.filtrovaneZaznamy = filtrovaneZaznamy;
    }
    
    public List<Kategoria> getSearchKategorie() {
        return searchKategorie;
    }

    public void setSearchKategorie(List<Kategoria> searchKategorie) {
        this.searchKategorie = searchKategorie;
    }

    public List<Kategoria> getMozneKategorie() {
        ArrayList<Kategoria> list = new ArrayList(getZoznamKategorii());
        for (Kategoria k: searchKategorie)
            list.remove(k);
        mozneKategorie = list;
        return list;
    }

    public void setMozneKategorie(List<Kategoria> mozneKategorie) {
        this.mozneKategorie = mozneKategorie;
    }

    public int getDeleteID() {
        return deleteID;
    }

    public void setDeleteID(int deleteID) {
        this.deleteID = deleteID;
    }
    
    public int getNewID() {
        return newID;
    }

    public void setNewID(int newID) {
        this.newID = newID;
    }

    public String getNewLabel() {
        return newLabel;
    }

    public void setNewLabel(String text) {
        this.newLabel = text;
    }

    public Zaznam getViewZaznam() {
        return viewZaznam;
    }

    public void setViewZaznam(Zaznam viewZaznam) {
        this.viewZaznam = viewZaznam;
    }

    public Kategoria getViewKategoria() {
        return viewKategoria;
    }

    public void setViewKategoria(Kategoria viewKategoria) {
        this.viewKategoria = viewKategoria;
    }

    public void pridajZaznam() {
        if (newLabel == null || newLabel.equals("")) {
            return;
        }

        Zaznam novyZaznam = new Zaznam();
        novyZaznam.setLabel(newLabel);
        persist(novyZaznam);
        newLabel = "";
    }

    public List<Kategoria> getZoznamKategorii() {
        Query query = entityManager.createNamedQuery("Kategoria.findAll");
        zoznamKategorii = query.getResultList();
        return zoznamKategorii;
    }

    public void setZoznamKategorii(List<Kategoria> zoznamKategorii) {
        this.zoznamKategorii = zoznamKategorii;
    }

    public String otvorZaznam(int id) {

        Query query = entityManager.createNamedQuery("Zaznam.findById");
        query.setParameter("id", id);
        viewZaznam = (Zaznam) query.getSingleResult();
        return "zaznam";
    }
   

    public void setZoznamZaznamov(List<Zaznam> zoznamZaznamov) {
        this.zoznamZaznamov = zoznamZaznamov;
    }

    public List<Zaznam> getZoznamZaznamov() {
        Query query = entityManager.createNamedQuery("Zaznam.findAll");
        zoznamZaznamov = query.getResultList();
        return zoznamZaznamov;
    }
    
    public void search(){
        ArrayList<ArrayList<Kategoria>> kategorie = new ArrayList();
        for (Kategoria k: getSearchKategorie()){
            ArrayList<Kategoria> subkategorie = new ArrayList(k.getKategoriaCollection());
            subkategorie.add(k);
            kategorie.add(subkategorie);
        }
        
        filtrovaneZaznamy = new ArrayList();
        for (Zaznam z: getZoznamZaznamov()){
            if (zaznamSplna(z, kategorie)) 
                filtrovaneZaznamy.add(z);
        }          
    }
    
    public boolean zaznamSplna(Zaznam zaznam, ArrayList<ArrayList<Kategoria>> kategorie){
        Collection<Kategoria> zaznamKategorie = zaznam.getKategoriaCollection();

        for (ArrayList<Kategoria> keyword: kategorie){
            boolean splna = false;                    
            for (Kategoria k: keyword){
                if (zaznamKategorie.contains(k)){
                    splna = true;
                    break;
                }
            }
            if (!splna) return false;
        }
        
        return true;
    }
    
    
    public void vymazSearchKategoriu(){
        Kategoria k = entityManager.getReference(Kategoria.class, deleteID);
        if (searchKategorie.contains(k)){
            searchKategorie.remove(k);
        }        
        deleteID = 0;
        newID = 0;
        filtrovaneZaznamy.clear();
    }
    
    public void vymazSearchKategorie(){
        searchKategorie.clear();
        filtrovaneZaznamy.clear();
        deleteID = 0;
        newID = 0;
    }
    
    public void pridajSearchKategoriu(){
        if (newID < 1) return;
        Kategoria k = entityManager.getReference(Kategoria.class, newID);
        searchKategorie.add(k);     
        deleteID = 0;
        newID = 0;
        filtrovaneZaznamy.clear();
    }

    public List<KategoriaIntend> formatovaneKategorie() {
        String text = "SELECT * "
                + "FROM Kategoria k "
                + "WHERE 0 = (SELECT count(*) "
                + "           FROM HierarchiaKategorii kz "
                + "           WHERE k.id = kz.id_podkategoria)";

        List<Kategoria> kategorie = (List<Kategoria>) entityManager.createNativeQuery(text, Kategoria.class).getResultList();
        List<KategoriaIntend> result = new ArrayList<KategoriaIntend>();
        for (Kategoria k : kategorie) {
            KategoriaIntend ki = new KategoriaIntend(k, 0);
            result.add(ki);
            result = formatovaneKategorieZKategorie(ki, result);
        }
        return result;
    }

    private List<KategoriaIntend> formatovaneKategorieZKategorie(KategoriaIntend ki, List<KategoriaIntend> result) {
        String text = "SELECT * "
                + "FROM Kategoria k, HierarchiaKategorii hk "
                + "WHERE k.id = hk.id_podkategoria AND hk.id_nadkategoria = " + ki.kategoria.getId();

        List<Kategoria> kategorie = (List<Kategoria>) entityManager.createNativeQuery(text, Kategoria.class).getResultList();
        for (Kategoria k : kategorie) {
            KategoriaIntend nki = new KategoriaIntend(k, ki.rank + 2);
            if (!result.contains(nki)) {
                result.add(nki);
                result = formatovaneKategorieZKategorie(nki, result);
            }
        }
        return result;
    }

    public List<Kategoria> zoznamNepouzitychKategori(int idZaznamu) {
        String text = "SELECT * \n"
                + "FROM Kategoria k \n"
                + "WHERE k.id not IN (SELECT kz.id_kategoria  \n"
                + "                   FROM Kategoria_zaznam kz \n"
                + "                   WHERE k.id = kz.id_kategoria AND kz.id_zaznam = " + idZaznamu + ")";

        List<Kategoria> kategorie = (List<Kategoria>) entityManager.createNativeQuery(text, Kategoria.class).getResultList();

        return kategorie;
    }

    public void odstranKategoriuZoznamu(int id) {
        Kategoria kategoria = entityManager.getReference(Kategoria.class, id);
        try {
            utx.begin();
            viewZaznam.getKategoriaCollection().remove(kategoria);
            kategoria.getZaznamCollection().remove(viewZaznam);
            entityManager.merge(kategoria);
            entityManager.merge(viewZaznam);
            entityManager.flush();
            utx.commit();
        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "exception caught", e);
            throw new RuntimeException(e);
        }
    }

    public void pridajKategoriuZaznamu() {
        if (newID == 0) {
            return;
        }
        Kategoria kategoria = entityManager.getReference(Kategoria.class, newID);
        newID = 0;
        try {
            utx.begin();
            viewZaznam.getKategoriaCollection().add(kategoria);
            kategoria.getZaznamCollection().add(viewZaznam);
            entityManager.merge(kategoria);
            entityManager.merge(viewZaznam);
            entityManager.flush();
            utx.commit();
        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "exception caught", e);
            throw new RuntimeException(e);
        }
    }

    public void vymazKategoriu() {
        if (deleteID < 1) {
            return;
        }
        Kategoria kategoria = entityManager.getReference(Kategoria.class, deleteID);
        try {
            utx.begin();
            kategoria = entityManager.merge(kategoria);
            for (Zaznam z: kategoria.getZaznamCollection()){
                z.getKategoriaCollection().remove(kategoria);
                entityManager.merge(z);
            }
            for (Kategoria k: kategoria.getKategoriaCollection()){
                k.getKategoriaCollection1().remove(kategoria);
                entityManager.merge(k);
            }
            for (Kategoria k: kategoria.getKategoriaCollection1()){
                k.getKategoriaCollection().remove(kategoria);
                entityManager.merge(k);
            }
            entityManager.remove(kategoria);
            entityManager.flush();
            utx.commit();
        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "exception caught", e);
            throw new RuntimeException(e);
        } 
        deleteID = 0;
        zoznamZaznamov = getZoznamZaznamov();
        zoznamKategorii = getZoznamKategorii();
    }

    public String vymazZaznam() {
        if (viewZaznam == null) {
            return "index";
        }
        try {
            utx.begin();
            viewZaznam = entityManager.merge(viewZaznam);
            entityManager.remove(viewZaznam);
            utx.commit();
            viewZaznam = null;
        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "exception caught", e);
            throw new RuntimeException(e);
        }
        return "index";
    }

    public void pridajPodkategoriu() {
        if (newLabel == null || newLabel.equals("")) {
            return;
        }

        Kategoria kategoria = new Kategoria();
        kategoria.setLabel(newLabel);
        persist(kategoria);
        try {
            utx.begin();
            System.out.println("id "+kategoria.getId());
            System.out.println("id2 "+newID);
            if (newID > 0) {
                Query query = entityManager.createNamedQuery("Kategoria.findById");
                query.setParameter("id", newID);
                Kategoria nadKategoria = (Kategoria) query.getSingleResult();
                
                kategoria.setKategoriaCollection1(nadKategoria.getKategoriaCollection1());
                kategoria.getKategoriaCollection1().add(nadKategoria);
                for (Kategoria k : kategoria.getKategoriaCollection1()) {              
                    System.out.println(k.getId());
                    k.getKategoriaCollection().add(kategoria);                 
                }
                entityManager.merge(kategoria);
            }
            entityManager.flush();
            utx.commit();
        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "exception caught", e);
            throw new RuntimeException(e);
        }
        newLabel = "";
        newID = 0;
    }

    public void persist(Object object) {
        try {
            utx.begin();
            entityManager.persist(object);
            entityManager.flush();
            utx.commit();
        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "exception caught", e);
            throw new RuntimeException(e);
        }
    }

}
